@@ -27,6 +27,7 @@ def get_commands_from_pkg(pkg) -> dict:
         dict: [description]
     """
     keep_groups = [
+        "osducli.commands.legal",
         "osducli.commands.list",
         "osducli.commands.unit",
         "osducli.commands.schema",
@@ -112,7 +113,9 @@ def cli(ctx):
     `___'[___/|___/ \__|
 
     Welcome to the OSDU CLI!
-    Note: This is currently a work in progress. Please share ideas / issues on the git page.
+
+    Note: This is currently a work in progress and may contain bugs or breaking changes.
+    Please share ideas / issues on the git page.
 
     Use `osducli version` to display the current version.
 
