@@ -34,8 +34,11 @@ logger = get_logger(__name__)
     "-rl",
     "--runid-log",
     help="Path to a file containing run ids to get status of (see dataload ingest -h).",
+    type=click.Path(exists=True, file_okay=True, readable=True, resolve_path=True),
+)
+@click.option(
+    "-w", "--wait", help="Whether to wait for runs to complete.", is_flag=True, show_default=True
 )
-@click.option("-w", "--wait", help="Whether to wait for runs to complete.", is_flag=True)
 @handle_cli_exceptions
 @command_with_output(None)
 def _click_command(state: State, runid: str = None, runid_log: str = None, wait: bool = False):
@@ -43,7 +46,66 @@ def _click_command(state: State, runid: str = None, runid_log: str = None, wait:
     return status(state, runid, runid_log, wait)
 
 
-def _status_check(config: CLIConfig, run_id_list: list):
+def status(state: State, runid: str = None, runid_log: str = None, wait: bool = False) -> dict:
+    """Get status of workflow runs
+
+    Args:
+        state (State): Global state
+        group (str): Email address of the group
+
+    Returns:
+        dict: Response from service
+    """
+    runids = []
+    if runid is not None:
+        runids = [runid]
+    elif runid_log is not None:
+        with open(runid_log) as handle:
+            runids = [run_id.rstrip() for run_id in handle]
+    else:
+        logger.error("Specify either runid or runid_log")
+        sys.exit(1)
+
+    return check_status(state.config, runids, wait)
+
+
+def check_status(config: CLIConfig, runids: list, wait: bool) -> list:
+    """Check statis for a list of runids
+
+    Args:
+        config (CLIConfig): configuration
+        runids (list): list of runids
+        wait (bool): whether to wait for status to change out of running
+
+    Returns:
+        list: list containing runid and status.
+    """
+    results = _check_status(config, runids)
+
+    if wait:
+        # parse the results to see if the ingestion is complete.
+        while True:
+            ingestion_complete = True
+            for result in results:
+                if result.get("status") == "running":
+                    ingestion_complete = False
+                    logger.debug(
+                        "Not all records finished. Checking again in 30s. Tried upto %s and found running",
+                        result.get("runId"),
+                    )
+                    break
+
+            if ingestion_complete:
+                break
+
+            print(results)
+            time.sleep(30)  # 30 seconds sleep.
+            results = _check_status(config, runids)  # recheck the status.
+
+    return results
+
+
+def _check_status(config: CLIConfig, run_id_list: list):
     logger.debug("list of run-ids: %s", run_id_list)
 
     results = []
@@ -69,49 +131,4 @@ def _status_check(config: CLIConfig, run_id_list: list):
                 )
         else:
             results.append({RUN_ID: run_id, STATUS: "Unable To fetch status"})
-    print(results)
-    return results
-
-
-def status(state: State, runid: str = None, runid_log: str = None, wait: bool = False) -> dict:
-    """Get status of workflow runs
-
-    Args:
-        state (State): Global state
-        group (str): Email address of the group
-
-    Returns:
-        dict: Response from service
-    """
-    runids = []
-    if runid is not None:
-        runids = [runid]
-    elif runid_log is not None:
-        with open(runid_log) as handle:
-            runids = [run_id.rstrip() for run_id in handle]
-    else:
-        logger.error("Specify either runid or runid_log")
-        sys.exit(1)
-
-    results = _status_check(state.config, runids)
-
-    if wait:
-        # parse the results to see if the ingestion is complete.
-        while True:
-            ingestion_complete = True
-            for result in results:
-                if result.get("status") == "running":
-                    ingestion_complete = False
-                    logger.debug(
-                        "Not all records finished. Checking again in 60s. Tried upto %s and found running",
-                        result.get("RunId"),
-                    )
-                    break
-
-            if ingestion_complete:
-                break
-
-            time.sleep(60)  # 60 seconds sleep.
-            results = _status_check(state.config, runids)  # recheck the status.
-
     return results
