@@ -15,16 +15,17 @@ from osducli.cliclient import CliOsduClient, handle_cli_exceptions
 
 # click entry point
 @click.command()
+@click.option("-k", "--kind", "kind", help="kind to search for")
 @click.option("-id", "--id", "_id", help="id to search for")
 @handle_cli_exceptions
 @command_with_output(None)
-def _click_command(state: State, _id: str):
+def _click_command(state: State, kind: str, _id: str):
     """Query search service"""
-    return query(state, _id)
+    return query(state, kind, _id)
 
 
-def query(state: State, id: str):  # TO FIX later pylint: disable=invalid-name,redefined-builtin
-    """[summary]
+def query(state: State, kind: str, id: str):  # pylint: disable=invalid-name,redefined-builtin
+    """Query search service
 
     Args:
         state (State): Global state
@@ -32,6 +33,6 @@ def query(state: State, id: str):  # TO FIX later pylint: disable=invalid-name,r
     connection = CliOsduClient(state.config)
 
     search_client = SearchClient(connection)
-    json_response = search_client.query_by_id(id)
+    json_response = search_client.query(kind, id)
 
     return json_response
